import os
import sys
import socket
import xmlrpclib
from ssl import SSLError
from getpass import getpass
import keyring
from sixfeetup.todo.ticket import Ticket

TRUISMS = ['y', 'yes', 'true']


class ToDo(object):
    """An object that represents the users list
    """

    def __init__(self, component=""):
        """
        """
        self.component = component
        self.server = None
        self.tickets = []
        if self.component:
            self.query_string = (
                "status!=closed&owner=%(username)s&component=%(component)s")
        else:
            self.query_string = (
                "status!=closed&owner=%(username)s")

    def initialize_server(self):
        server_string = (
            "https://%s:%s@trac.sixfeetup.com/operations/login/xmlrpc")
        self.server = xmlrpclib.ServerProxy(
            server_string % (self.login, self.password))
        # let's test that the user has access to the XML-RPC server and
        # that they have the permission to use the XML-RPC server
        try:
            self.server.system.getAPIVersion()
        except xmlrpclib.ProtocolError:
            print (
                "\nPlease check that the username and password that were "
                "entered are correct and try again")
            sys.exit(1)
        except xmlrpclib.Fault, e:
            print (
                "\n%s\n\nYou may not have permission to access the XML-RPC "
                "server on the operations Trac instance. Check with a "
                "pirate" % e)
            sys.exit(1)
        # you've got five seconds to complete a task, come on Trac!
        socket.setdefaulttimeout(5)

    def user_credentials(self, login=None, username=None):
        """
        """
        if username is None:
            username = os.getenv("USER", "")
            # if there is no USER env var...
            if username:
                print "You must supply a username (-u)"
                sys.exit(1)
        self.login = login
        if not self.login:
            self.login = username
        self.username = username
        self.password = keyring.get_password('todo', self.login)
        if self.password is None:
            self.password = getpass("Trac Password: ")
            #if not keyring.supported():
            #    return
            save_password = raw_input(
                "Save this password in your keychain? [y/N] ")
            if save_password.lower() in TRUISMS:
                keyring.set_password('todo', self.login, self.password)

    def get_tickets(self):
        query_options = dict(username=self.username, component=self.component)
        ticket_ids = self.server.ticket.query(
            self.query_string % query_options)
        ticket_priorities = self.server.ticket.priority.getAll()
        for ticket_id in ticket_ids:
            # for some reason, some tickets are impossible to retrieve
            # but this is completely random?!?
            try:
                ticket = self.server.ticket.get(ticket_id)
            except SSLError:
                error_msg = "couldn't retrieve #%s" % ticket_id
                ticket_obj = Ticket(
                    ticket_id, error_msg, "!ERROR - timed out", "???", "none")
                self.tickets.append(ticket_obj)
                continue
            ticket_dict = ticket[3]
            milestone = ticket_dict['milestone']
            summary = ticket_dict['summary']
            flag = ticket_dict['flag']
            priority = ticket_dict['priority']
            time_remaining = ticket_dict['tt_remaining']
            ticket_obj = Ticket(
                ticket_id,
                summary,
                milestone,
                priority,
                flag,
                time_remaining,
                priorities=ticket_priorities,
            )
            self.tickets.append(ticket_obj)
        # let's sort them once we are done
        self.tickets.sort()

    def show_list(self):
        """This is kind of fugly at the moment, could use a little love
        """
        milestone = ""
        previous_flag = -1
        for ticket in self.tickets:
            if milestone != ticket.milestone:
                previous_flag = -1
                print
                print " ## %s ##" % ticket.milestone
            milestone = ticket.milestone
            priority = ticket.priority
            flag = ticket.flag
            if str(flag).lower() == "none":
                flag = ""
            else:
                # XXX: we assume all flags are days...
                flag = "%s " % flag[3:]
            summary = ticket.summary
            if previous_flag != flag:
                print
            previous_flag = flag
            rem = " "
            if not ticket.time_remaining or ticket.time_remaining == "0":
                rem = "*"
            print "%(rem)s%(flag)s(%(priority)s): %(summary)s" % locals()
